package sam.machine.active ;

import sam.machine.lang.* ;
import java.util.* ; 

/**   
  * ActiveTheory determined by a sam.machine.lang.Theory
  * @version 4/6/2023 
   */
public class ActiveTheory  { 
   public Theory theory ;          // the source program 
   public int initialSize ;        // the # of source program

   // The compiled active program 
   protected Vector<ActiveRule> defRules ;   // all definite ActiveRule(s)
   protected Vector<ActiveRule> exiRules ;   // all existential ActiveRule(s)
   protected Vector<ActiveRewrite> allRewrites ; // all ActiveRewrite(s)
   // other data later ... 

   /**
     *  Construct initial ActiveTheory from source program. 
     *  InferenceManager apply active inferences in stages:
     *    - Saturate active definite rules and active rewrites
     *    - then apply an existential rule in a limited manner.
      */
   public ActiveTheory(Theory theory) { 
      this.initialSize = theory.size() ;    
      int indx = 1 ;   
      Iterator it = theory.iterator() ; 
      while (it.hasNext()) { 
         Object next = it.next() ;
         if (next instanceof Rule) {   
            Rule rule = (Rule)next ; 
            ActiveRule arule = new ActiveRule(rule) ;  
            arule.index = indx ;
            indx++ ;       
            if (rule.existential>0) {   // SHOULD BE!
               if (exiRules==null) exiRules = new Vector<ActiveRule>() ;
               exiRules.add(arule) ;
            }
            else { // arule is definite rule
               if (defRules==null) defRules = new Vector<ActiveRule>() ;
               defRules.add(arule) ; 
            }
         }
         else { // next is a static program rewrite 
            //Rewrite rw = (Rewrite)(it.next()) ;         //### stupid
            Rewrite rw = (Rewrite)next ; 
            ActiveRewrite arw = new ActiveRewrite(rw) ; 
            arw.index = indx ;
            indx++ ; 
            if (allRewrites==null) allRewrites = new Vector<ActiveRewrite>() ;       
            allRewrites.add(arw) ;
         }
      }
      if (defRules!=null) defRules.trimToSize() ;
      if (exiRules!=null) exiRules.trimToSize() ;
      /* allRewrites is a dynamic component: rule consequents can add 
         new ActiveRewrite(Functor left,Functor right) to be used in
         in subsequent inference stages. */
   }
   
   public String report() { 
      StringBuffer buff = new StringBuffer() ; 
      if (defRules!=null) { // report defRules
         buff.append("definite active rules\n") ; 
         Iterator dit = defRules.iterator() ;
         while (dit.hasNext()) {   
            ActiveRule ar = (ActiveRule)(dit.next()) ;
            buff.append(" "+ar.index +" "+ar.rule.toString()+"\n") ;
         } 
      }
      if (exiRules!=null) { // report exiRules
         buff.append("existential active rules\n") ; 
         Iterator eit = exiRules.iterator() ;
         while (eit.hasNext()) {   
            ActiveRule ar = (ActiveRule)(eit.next()) ;
            buff.append(" "+ar.index +" "+ar.rule.toString()+"\n") ;
         } 
      }
      if (allRewrites!=null) { // report allRewrites
         buff.append("all active rewrites\n") ; 
         Iterator ait = allRewrites.iterator() ;
         while (ait.hasNext()) {   
            ActiveRewrite arw = (ActiveRewrite)(ait.next()) ;
            buff.append(" "+arw.index +" "+ (arw.dynamic ? "dynamic" : "static") +" rewrite \n") ;
         } 
      }
      return buff.toString() ;
   }

   /**
     * preliminary tests ...
      */
   public static void main(String[] args) {
   }

} // end ActiveTheory

/* end notes
1. Modify active.DGM re current AT.
2. A dynamic ActiveRewrite has index i where Activetree branch[i] 
   is the a grounded  equality literal -- for use in inference reports.  
*/

