package sam.machine.active ;
import sam.machine.lang.* ;

/**
  *  ActiveTree
  *  Skolem Machine active inference management 
  *  Inspired by colog14I/sam.infer.Tree 
  *  All components are public to enable spying on action.
  *  @version 4/8/2023
   */
public class ActiveTree implements Runnable {
   public Theory theory ;             // user theory/program
   public ActiveTheory activetheory ; // dynamic 
   public boolean log ;  // logging flag
   protected Functor[] branch ;
   public int depth ;        // restricts depth of branch
   public int width ;        // restrict width of inference tree

   ///////////////////////////////////////////////////////////////////
   // logging data structures needed as in colog14/sam/infer/Tree.java.
   // Logging data structures used by TreePane to draw inference tree.
   ///////////////////////////////////////////////////////////////////

   /* 
    * Set logging true for AutoLog Viewer
    * Set logging false for command line use
     */
   public void setLogging(boolean b) { 
      this.log = b ;
   }


   /* 
    * Construct an ActiveTree ready for setTheory(-)
    * 
     */
   public ActiveTree(int depth, int width) { 
      this.depth = depth ;
      this.width = width ;
   }   

   /**
     *  Set the active theory. 
     *  Called from sam/gui/Autolog.java.
     */
   public void setTheory(Theory thry) { 
      this.theory = thry ; 
      this.activetheory = new ActiveTheory(thry) ; 
      // this.atLeaf = -1 ; // turn off inspection ???
   }

   /**
     * Marshal concurrent staged activetheory inferences
      */
   public void run() { 
      // UNDER CONSTRUCTION
   }

   /**
     * Debug setTheory(-) for ActiveTree
      */
   public static void main(String[] args) { 
      // a test autolog program Theory
      String thy = 
         "true=>p(Z). f(X)=X. true=>p(a). p(X)=>q(X)." ;
      Theory theory = Theory.translate(thy) ; 
      AutoLogAnalyzer.analyze(theory) ;
      System.out.println(theory) ; 
      System.out.println("--------------------") ; 
      ActiveTree atree = new ActiveTree(10,10) ; 
      atree.setTheory(theory) ;
      // spy on atree.activetheory
      System.out.println(atree.activetheory.report()) ; 
   }

}

/* dev notes
   ActiveTree borrows ideas from colog14I/sam/infer/Tree.
   ActiveTree marshals concurrent inferences for a current branch, 
   and then works on other branches which arose due to disjuctive
   consequent inferences on a previous branch. ActiveTree will log 
   active branches for TreePane when ActiveTree works in the 
   sam.gui.search.AutoLog viewer. NOTE: colog14I/gui/TreePane.java 
   DOES refer to colog14I/infer/Tree.java


   autolog/sam/gui/search/AutoLog.java still has some code
   borrowed from colog14I/sam.gui.Colog.java, some commented out. 
   NOTE: colog14I/sam.gui.Colog.java DOES refer to TreePane.
   See, e.g., colog14I/sam.gui.Colog.java code commented-out as  
   "//TP" and "//Tree". NOTE: colog14I/infer/Tree.java does NOT refer
   to colog14I/gui/TreePane.java. 


    
*/

